@@ -1,6 +1,6 @@
 package Statocles::App::Blog;
 # ABSTRACT: A blog application
-$Statocles::App::Blog::VERSION = '0.043';
+$Statocles::App::Blog::VERSION = '0.044';
 use Statocles::Base 'Class';
 use Getopt::Long qw( GetOptionsFromArray );
 use Statocles::Store::File;
@@ -72,8 +72,28 @@ sub command {
             'date:s',
         );
 
-        my $title = join " ", @argv[1..$#argv];
-        if ( !$ENV{EDITOR} && !$title ) {
+        my %doc = (
+            %$default_post,
+            title => join " ", @argv[1..$#argv],
+        );
+
+        # Read post content on STDIN
+        if ( !-t *STDIN ) {
+            my $content = do { local $/; <STDIN> };
+            %doc = (
+                %doc,
+                $self->store->parse_frontmatter( "<STDIN>", $content ),
+            );
+
+            # Re-open STDIN as the TTY so that the editor (vim) can use it
+            # XXX Is this also a problem on Windows?
+            if ( -e '/dev/tty' ) {
+                close STDIN;
+                open STDIN, '/dev/tty';
+            }
+        }
+
+        if ( !$ENV{EDITOR} && !$doc{title} ) {
             say STDERR <<"ENDHELP";
 Title is required when \$EDITOR is not set.
 
@@ -98,36 +118,17 @@ ENDHELP
             sprintf( '%02i', $day ),
         );
 
-        my %doc = (
-            %$default_post,
-            title => $title,
-        );
-
-        # Read post content on STDIN
-        if ( !-t *STDIN ) {
-            $doc{content} = do { local $/; <STDIN> };
-            # Re-open STDIN as the TTY so that the editor (vim) can use it
-            # XXX Is this also a problem on Windows?
-            if ( -e '/dev/tty' ) {
-                close STDIN;
-                open STDIN, '/dev/tty';
-            }
-        }
-
         if ( $ENV{EDITOR} ) {
             # I can see no good way to test this automatically
-            my $slug = lc $title || "new post";
-            $slug =~ s/\s+/-/g;
+            my $slug = $self->make_slug( $doc{title} || "new post" );
             my $path = Path::Tiny->new( @date_parts, $slug, "index.markdown" );
             my $tmp_path = $self->store->write_document( $path => \%doc );
             system $ENV{EDITOR}, $tmp_path;
             %doc = %{ $self->store->read_document( $path ) };
             $self->store->path->child( $path )->remove;
-            $title = $doc{title};
         }
 
-        my $slug = lc $title;
-        $slug =~ s/\s+/-/g;
+        my $slug = $self->make_slug( $doc{title} );
         my $path = Path::Tiny->new( @date_parts, $slug, "index.markdown" );
         my $full_path = $self->store->write_document( $path => \%doc );
         say "New post at: $full_path";
@@ -143,43 +144,67 @@ ENDHELP
 }
 
 
+sub make_slug {
+    my ( $self, $slug ) = @_;
+    $slug =~ s/[\W]+/-/g;
+    return lc $slug;
+}
+
+
 sub post_pages {
     my ( $self ) = @_;
-    my $today = Time::Piece->new->ymd;
-    my @pages;
-    for my $doc ( @{ $self->store->documents } ) {
-        my $path = $doc->path;
-        $path =~ s{/{2,}}{/}g;
-        $path =~ s{[.]\w+$}{.html};
+    my @pages = map { $self->_make_post_page( $_ ) } $self->_sorted_docs;
+    $self->_post_pages( [ @pages ] );
+    return @pages;
+}
 
-        my @date_parts = $path =~ m{/(\d{4})/(\d{2})/(\d{2})/[^/]+(?:/index[.]html)?$};
+# Return the post docs sorted by date, pruning any docs that are after
+# the current date
+sub _sorted_docs {
+    my ( $self ) = @_;
+
+    my $today = Time::Piece->new->ymd;
+    my @doc_dates;
+    for my $doc ( @{ $self->store->documents } ) {
+        my @date_parts = $doc->path =~ m{/(\d{4})/(\d{2})/(\d{2})/[^/]+(?:/index[.]markdown)?$};
         next unless @date_parts;
         my $date = join "-", @date_parts;
 
         next if $date gt $today;
 
-        my @tags;
-        for my $tag ( @{ $doc->tags } ) {
-            push @tags, $self->link(
-                text => $tag,
-                href => join( "/", 'tag', $self->_tag_url( $tag ) ),
-            );
-        }
+        push @doc_dates, [ $doc, $date ];
+    }
 
-        push @pages, Statocles::Page::Document->new(
-            app => $self,
-            layout => $self->site->theme->template( site => 'layout.html' ),
-            template => $self->site->theme->template( blog => 'post.html' ),
-            document => $doc,
-            path => $path,
-            date => $doc->has_date ? $doc->date : Time::Piece->strptime( $date, '%Y-%m-%d' ),
-            tags => \@tags,
+    return map { $_->[0] } sort { $b->[1] cmp $a->[1] } @doc_dates;
+}
+
+sub _make_post_page {
+    my ( $self, $doc ) = @_;
+
+    my $path = $doc->path;
+    $path =~ s{/{2,}}{/}g;
+    $path =~ s{[.]\w+$}{.html};
+
+    my @date_parts = $doc->path =~ m{/(\d{4})/(\d{2})/(\d{2})/[^/]+(?:/index[.]markdown)?$};
+    my $date = join "-", @date_parts;
+
+    my @tags;
+    for my $tag ( @{ $doc->tags } ) {
+        push @tags, $self->link(
+            text => $tag,
+            href => join( "/", 'tag', $self->_tag_url( $tag ) ),
         );
     }
 
-    $self->_post_pages( [ @pages ] );
-
-    return @pages;
+    return Statocles::Page::Document->new(
+        app => $self,
+        layout => $self->site->theme->template( site => 'layout.html' ),
+        template => $self->site->theme->template( blog => 'post.html' ),
+        document => $doc,
+        path => $path,
+        date => $doc->has_date ? $doc->date : Time::Piece->strptime( $date, '%Y-%m-%d' ),
+        tags => \@tags,
+    );
 }
 
 
@@ -340,6 +365,22 @@ sub _tag_url {
 }
 
 
+sub recent_posts {
+    my ( $self, $count ) = @_;
+
+    my $today = Time::Piece->new->ymd;
+    my @pages;
+    my @docs = $self->_sorted_docs;
+    for my $doc ( @docs[0..$count-1] ) {
+        my $page = $self->_make_post_page( $doc );
+        $page->path( join "/", $self->url_root, $page->path );
+        push @pages, $page;
+    }
+
+    return @pages;
+}
+
+
 # XXX This is TERRIBLE. We need to do this better. Perhaps a "url()" helper in the
 # template? And a full_url() helper? Or perhaps the template knows whether it should
 # use absolute (/whatever) or full (http://www.example.com/whatever) URLs?
@@ -363,7 +404,7 @@ Statocles::App::Blog - A blog application
 
 =head1 VERSION
 
-version 0.043
+version 0.044
 
 =head1 DESCRIPTION
 
@@ -436,6 +477,10 @@ So, given a document with tags "foo", and "bar":
 Run a command on this app. The app name is used to build the help, so
 users get exactly what they need to run.
 
+=head2 make_slug( $title )
+
+Given a post title, remove special characters to create a slug.
+
 =head2 post_pages()
 
 Get the individual post Statocles::Page objects.
@@ -461,6 +506,11 @@ tag links. The common attributes are:
     text => 'The tag text'
     href => 'The URL to the tag page'
 
+=head2 recent_posts( $count )
+
+Get the last $count recent posts for this blog. Useful for templates and site
+index pages.
+
 =head2 page_url( page )
 
 Return the absolute URL to this page, removing the "/index.html" if necessary.
