@@ -1,6 +1,6 @@
 package Statocles::Site;
 # ABSTRACT: An entire, configured website
-$Statocles::Site::VERSION = '0.043';
+$Statocles::Site::VERSION = '0.044';
 use Statocles::Base 'Class', 'Emitter';
 use Scalar::Util qw( blessed );
 use Text::Markdown;
@@ -129,6 +129,11 @@ has _write_deploy => (
 
 sub BUILD {
     my ( $self ) = @_;
+
+    if ( $self->index && !$self->app( $self->index ) ) {
+        die sprintf 'ERROR: Index app "%s" does not exist', $self->index;
+    }
+
     $Statocles::SITE = $self;
     for my $app ( values %{ $self->apps } ) {
         $app->site( $self );
@@ -320,7 +325,7 @@ Statocles::Site - An entire, configured website
 
 =head1 VERSION
 
-version 0.043
+version 0.044
 
 =head1 SYNOPSIS
 
