@@ -1,6 +1,6 @@
 package Statocles::Store::File;
 # ABSTRACT: A store made up of plain files
-$Statocles::Store::File::VERSION = '0.043';
+$Statocles::Store::File::VERSION = '0.044';
 use Statocles::Base 'Class';
 with 'Statocles::Store';
 use Scalar::Util qw( weaken blessed );
@@ -73,8 +73,7 @@ sub read_documents {
         next unless $self->_is_owned_path( $path );
         if ( $path =~ /[.]markdown$/ ) {
             my $rel_path = rootdir->child( $path->relative( $root_path ) );
-            my $data = $self->read_document( $rel_path );
-            push @docs, Statocles::Document->new( path => $rel_path, %$data );
+            push @docs, $self->read_document( $rel_path );
         }
     }
     return \@docs;
@@ -100,15 +99,16 @@ sub read_document {
     my ( $self, $path ) = @_;
     site->log->debug( "Read document: " . $path );
     my $full_path = $self->path->child( $path );
-    my $doc = $self->_parse_frontmatter( $full_path );
-    return $self->_thaw_document( $doc );
+    my %doc = $self->parse_frontmatter( $full_path, $full_path->slurp_utf8 );
+    return Statocles::Document->new( %doc, path => $path );
 }
 
-sub _parse_frontmatter {
-    my ( $self, $path ) = @_;
+
+sub parse_frontmatter {
+    my ( $self, $from, $content ) = @_;
     my $doc;
 
-    my @lines = $path->lines_utf8;
+    my @lines = split /\n/, $content;
     if ( $lines[0] =~ /^---/ ) {
         shift @lines;
 
@@ -117,28 +117,23 @@ sub _parse_frontmatter {
 
         # If we did not find the marker between YAML and Markdown
         if ( $i < 0 ) {
-            die "Could not find end of front matter (---) in '$path'\n";
+            die "Could not find end of front matter (---) in '$from'\n";
         }
 
         # Before the marker is YAML
         eval {
-            $doc = YAML::Load( join "", splice @lines, 0, $i );
+            $doc = YAML::Load( join "\n", splice( @lines, 0, $i ), "" );
         };
         if ( $@ ) {
-            die "Error parsing YAML in '$path'\n$@";
+            die "Error parsing YAML in '$from'\n$@";
         }
 
         # Remove the last '---' mark
         shift @lines;
     }
 
-    $doc->{content} = join "", @lines;
+    $doc->{content} = join "\n", @lines, "";
 
-    return $doc;
-}
-
-sub _thaw_document {
-    my ( $self, $doc ) = @_;
     if ( exists $doc->{date} ) {
 
         my $dt;
@@ -163,7 +158,8 @@ sub _thaw_document {
 
         $doc->{date} = $dt;
     }
-    return $doc;
+
+    return %$doc;
 }
 
 
@@ -188,9 +184,15 @@ sub write_document {
 
 sub _freeze_document {
     my ( $self, $doc ) = @_;
+    delete $doc->{path}; # Path should not be in the document
     if ( exists $doc->{date} ) {
         $doc->{date} = $doc->{date}->strftime( $DATETIME_FORMAT );
     }
+    if ( exists $doc->{links} ) {
+        if ( !keys %{ $doc->{links} } ) {
+            delete $doc->{links};
+        }
+    }
     return $doc;
 }
 
@@ -257,7 +259,7 @@ Statocles::Store::File - A store made up of plain files
 
 =head1 VERSION
 
-version 0.043
+version 0.044
 
 =head1 DESCRIPTION
 
@@ -299,8 +301,13 @@ Read the directory C<path> and create the L<document|Statocles::Document> object
 =head2 read_document( path )
 
 Read a single L<document|Statocles::Document> in Markdown with optional YAML
-frontmatter and return a datastructure suitable to be given to
-L<Statocles::Document|Statocles::Document>.
+frontmatter.
+
+=head2 parse_frontmatter( $from, $content )
+
+Parse a document with YAML frontmatter. $from is a string identifying where the
+content comes from (a path or other identifier). $content is the content to
+parse for frontmatter.
 
 =head2 write_document( $path, $doc )
 
